import { LitElement, nothing, TemplateResult } from 'lit';
import { InterfaceColor, InterfaceLook } from '@umbraco-ui/uui-base/lib/types';
export type UUIButtonState = undefined | 'waiting' | 'success' | 'failed';
export type UUIButtonType = 'submit' | 'button' | 'reset';
declare const UUIButtonElement_base: (new (...args: any[]) => import("@umbraco-ui/uui-base/lib/mixins").FormControlMixinInterface) & (new (...args: any[]) => import("@umbraco-ui/uui-base/lib/mixins").LabelMixinInterface) & (new (...args: any[]) => import("@umbraco-ui/uui-base/lib/mixins").PopoverTargetMixinInterface) & typeof LitElement;
/**
 *  @element uui-button
 *  @fires {UUIButtonEvent} click - fires when the element is clicked
 *  @slot - for default content
 *  @slot label - for label content
 *  @slot extra - for extra
 *  @description - All-around button
 *  @cssprop --uui-button-height - overwrite the button height
 *  @cssprop --uui-button-border-width - overwrite the border width
 *  @cssprop --uui-button-border-radius - overwrite the border radius
 *  @cssprop --uui-button-font-weight - overwrite the font weight
 *  @cssprop --uui-button-font-size - overwrite the font size
 *  @cssprop --uui-button-background-color - overwrite the background color
 *  @cssprop --uui-button-background-color-hover - overwrite the background color for hover state
 *  @cssprop --uui-button-background-color-disabled - overwrite the background color for disabled state
 *  @cssprop --uui-button-border-color - overwrite the border color
 *  @cssprop --uui-button-border-color-hover - overwrite the border color for hover state
 *  @cssprop --uui-button-border-color-disabled - overwrite the border color for disabled state
 *  @cssprop --uui-button-contrast - overwrite the text color
 *  @cssprop --uui-button-contrast-hover - overwrite the text color for hover state
 *  @cssprop --uui-button-contrast-disabled - overwrite the text color for disabled state
 *  @cssprop --uui-button-content-align - Overwrite justify-content alignment. Possible values: 'left', 'right', 'center'.
 */
export declare class UUIButtonElement extends UUIButtonElement_base {
    /**
     * Specifies the type of button
     * @type { "submit" | "button" | "reset" }
     * @attr
     * @default "button"
     */
    type: UUIButtonType;
    /**
     * Disables the button, changes the looks of it and prevents if from emitting the click event
     * @type {boolean}
     * @attr
     * @default false
     */
    disabled: boolean;
    /**
     * Changes the look of the button to one of the predefined, symbolic looks. For example - set this to positive if you want nice, green "confirm" button.
     * @type {"default" | "primary" | "secondary" | "outline" | "placeholder"}
     * @attr
     * @default "default"
     */
    look: InterfaceLook;
    /**
     * Changes the look of the button to one of the predefined, symbolic looks. For example - set this to positive if you want nice, green "confirm" button.
     * @type {"default" | "positive" | "warning" | "danger"}
     * @attr
     * @default "default"
     */
    color: InterfaceColor;
    /**
     * Makes the left and right padding of the button narrower.
     * @type {boolean}
     * @attr
     * @default false
     */
    compact: boolean;
    /**
     * Sets the state of the button. With waiting state a loader will show, the success state and fail states display icons. State is reset do default automatically after 3 seconds.
     * @type {undefined |'waiting' | 'success' | 'failed'}
     * @attr
     * @default undefined
     */
    state: UUIButtonState;
    /**
     * Set an href, this will turns the button into a anchor tag.
     * @type {string}
     * @attr
     * @default undefined
     */
    href?: string;
    /**
     * Set an anchor tag target, only used when using href.
     * @type {string}
     * @attr
     * @default undefined
     */
    target?: '_blank' | '_parent' | '_self' | '_top';
    protected _button: HTMLInputElement;
    constructor();
    protected getFormElement(): HTMLElement;
    private _onHostClick;
    private _resetStateTimeout?;
    updated(changedProperties: Map<string | number | symbol, unknown>): void;
    protected renderState(): TemplateResult | typeof nothing;
    render(): TemplateResult<1>;
    static styles: import("lit").CSSResult[];
}
declare global {
    interface HTMLElementTagNameMap {
        'uui-button': UUIButtonElement;
    }
}
export {};
